#!/usr/bin/env node
import {
  getCasingFunction,
  logger
} from "./chunk-6VVQCB2M.mjs";

// src/main.ts
import sourceMaps from "source-map-support";
import meow from "meow";
import fs3 from "fs-extra";
import path3 from "path";

// src/lib/commit-changes.ts
import simpleGit from "simple-git";
var git = simpleGit();
async function commitChanges(message) {
  await git.add("./*");
  await git.commit(message);
}

// src/lib/convert-files.ts
import fs2 from "fs-extra";
import path from "path";

// src/lib/get-files-to-process.ts
import fs from "fs-extra";
import { glob } from "glob";
import parseGitignore from "parse-gitignore";
var alwaysIgnorePatterns = [
  "node_modules/",
  ".git/",
  "package-lock.json",
  "npm-shrinkwrap.json",
  "pnpm-lock.yaml",
  "pnpm-workspace.yaml"
];
async function getFilesToProcess(directoryPath, gitignorePath, fileExtensions) {
  const gitignoreContent = await fs.readFile(gitignorePath, "utf-8");
  const ignoredPatterns = parseGitignore(gitignoreContent);
  const globIgnorePatterns = [
    ...alwaysIgnorePatterns,
    ...ignoredPatterns.patterns
  ].flatMap((pattern) => {
    pattern = pattern.replace(/^\//, "");
    const patterns = [pattern];
    if (!pattern.startsWith("**") && !pattern.endsWith("/")) {
      patterns.push(`**/${pattern}/**`);
    }
    if (pattern.endsWith("/")) {
      patterns.push(`**/${pattern}**`);
    }
    return patterns;
  });
  const globPatterns = fileExtensions && fileExtensions.length > 0 ? fileExtensions.map((ext) => `**/*.${ext}`) : ["**/*"];
  logger.debug("Glob patterns:", globPatterns);
  logger.debug("Ignored patterns:", globIgnorePatterns);
  const files = await glob(globPatterns, {
    cwd: directoryPath,
    ignore: globIgnorePatterns,
    absolute: true,
    nodir: true,
    dot: true
    // Include hidden files
  });
  return files;
}

// src/lib/convert-files.ts
var TEMP_SUFFIX = "__tmp";
async function renameFilesAndFolders(directoryPath, gitignorePath, phase, casingFn) {
  const absoluteDirectoryPath = path.resolve(directoryPath);
  logger.info(
    `Starting renameFilesAndFolders - Phase: ${phase}, Directory: ${absoluteDirectoryPath}`
  );
  const filesToProcess = await getFilesToProcess(
    absoluteDirectoryPath,
    gitignorePath
  );
  logger.info(`Found ${filesToProcess.length} files to process`);
  const renamedPaths = /* @__PURE__ */ new Set();
  for (const filePath of filesToProcess) {
    const newPath = phase === "phase1" ? getNewPathPhaseOne(filePath, absoluteDirectoryPath, casingFn) : getNewPathPhaseTwo(filePath, absoluteDirectoryPath);
    logger.debug(`${phase} - Old path: ${filePath}`);
    logger.debug(`${phase} - New path: ${newPath}`);
    if (newPath !== filePath) {
      await moveFile(filePath, newPath);
      renamedPaths.add(path.dirname(filePath));
    }
  }
  logger.info(`Removing empty directories after ${phase}`);
  await removeEmptyDirectories(absoluteDirectoryPath, renamedPaths);
}
async function removeEmptyDirectories(baseDirectory, renamedPaths) {
  const sortedPaths = Array.from(renamedPaths).sort(
    (a, b) => b.length - a.length
  );
  for (const dirPath of sortedPaths) {
    if (!dirPath.startsWith(baseDirectory)) {
      continue;
    }
    try {
      const files = await fs2.readdir(dirPath);
      if (files.length === 0) {
        await fs2.rmdir(dirPath);
        logger.debug(`Removed empty directory: ${dirPath}`);
      }
    } catch (error) {
      logger.error(`Error processing directory ${dirPath}: ${error}`);
    }
  }
}
function getNewPathPhaseOne(filePath, basePath, casingFn) {
  const relativePath = path.relative(basePath, filePath);
  const segments = relativePath.split(path.sep);
  const newSegments = segments.map((segment, index) => {
    if (index === segments.length - 1) {
      return convertFileName(segment, casingFn);
    }
    return convertSegment(segment, casingFn);
  });
  return path.join(basePath, ...newSegments);
}
function convertFileName(fileName, casingFn) {
  const ext = path.extname(fileName);
  const baseName = path.basename(fileName, ext);
  if (
    /** Preserve Next.js _app.tsx etc */
    baseName.startsWith("_") || /** Preserve files like README or LICENSE */
    baseName === baseName.toUpperCase()
  ) {
    return fileName;
  }
  const convertedBaseName = convertSegment(baseName, casingFn);
  return convertedBaseName + ext;
}
function convertSegment(segment, casingFn) {
  if (segment.startsWith("[")) {
    return segment;
  }
  const converted = casingFn(segment);
  if (converted === segment) {
    return converted;
  } else if (converted.toLowerCase() === segment.toLowerCase()) {
    return converted + TEMP_SUFFIX;
  } else {
    return converted;
  }
}
function getNewPathPhaseTwo(filePath, basePath) {
  const relativePath = path.relative(basePath, filePath);
  const segments = relativePath.split(path.sep);
  const newSegments = segments.map((segment, index) => {
    if (index === segments.length - 1) {
      const ext = path.extname(segment);
      const baseName = path.basename(segment, ext);
      if (baseName.endsWith(TEMP_SUFFIX)) {
        const newBaseName = baseName.slice(0, -TEMP_SUFFIX.length).toLowerCase();
        logger.debug(
          `Phase 2 conversion: ${baseName}${ext} -> ${newBaseName}${ext}`
        );
        return newBaseName + ext;
      }
    } else if (segment.endsWith(TEMP_SUFFIX)) {
      const newSegment = segment.slice(0, -TEMP_SUFFIX.length).toLowerCase();
      logger.debug(`Phase 2 conversion: ${segment} -> ${newSegment}`);
      return newSegment;
    } else if (segment.includes(TEMP_SUFFIX)) {
      logger.warn(`Unexpected TEMP_SUFFIX in middle of segment: ${segment}`);
    }
    return segment;
  });
  return path.join(basePath, ...newSegments);
}
async function moveFile(oldPath, newPath) {
  try {
    await fs2.ensureDir(path.dirname(newPath));
    await fs2.move(oldPath, newPath, { overwrite: false });
    logger.debug(`Moved: ${oldPath} -> ${newPath}`);
  } catch (error) {
    logger.error(
      `Failed to move file: ${oldPath} -> ${newPath}. Error: ${error}`
    );
  }
}

// src/lib/run-codemod.ts
import Runner from "jscodeshift/src/Runner.js";
import path2 from "node:path";
import { fileURLToPath } from "node:url";
async function runCodemod(directoryPath, gitignorePath, casingType) {
  logger.debug("Running codemod");
  logger.debug("Directory:", directoryPath);
  logger.debug("Gitignore path:", gitignorePath);
  logger.debug("Casing type:", casingType);
  const codeExtensions = ["ts", "tsx", "js", "jsx", "mjs", "cjs"];
  const inputFiles = await getFilesToProcess(
    directoryPath,
    gitignorePath,
    codeExtensions
  );
  logger.debug(`Found ${inputFiles.length} files to process`);
  const __filename2 = fileURLToPath(import.meta.url);
  const __dirname2 = path2.dirname(__filename2);
  const codemodPath = path2.resolve(
    __dirname2,
    casingType === "kebab" ? "./codemods/convert-import-export-kebab.cjs" : "./codemods/convert-import-export-snake.cjs"
  );
  logger.debug("Codemod path:", codemodPath);
  const result = await Runner.run(codemodPath, inputFiles, {
    parser: "tsx",
    // verbose: 1,
    // babel: true,
    extensions: codeExtensions.join(","),
    runInBand: true
  });
  return result;
}

// src/main.ts
sourceMaps.install();
var cli = meow(
  `
  Usage
    $ npx convert-codebase-casing <directory>

  Options
    --help, -h             Show help text
    --log-level, -l        Log level: 'error', 'warn', 'info', or 'debug' (default: 'info')
    --casing, -c           Casing type: 'kebab' or 'snake' (default: 'kebab')

  Examples
    $ npx convert-codebase-casing ./src
    $ npx convert-codebase-casing ./src --casing snake
    $ npx convert-codebase-casing ./src --log-level debug
`,
  {
    importMeta: import.meta,
    flags: {
      help: {
        type: "boolean",
        shortFlag: "h"
      },
      casing: {
        type: "string",
        shortFlag: "c",
        default: "kebab"
      },
      logLevel: {
        type: "string",
        shortFlag: "l",
        default: "info"
      }
    }
  }
);
logger.setLevel(cli.flags.logLevel);
async function run() {
  const directoryPath = cli.input[0];
  if (!directoryPath) {
    logger.error("Please specify a directory path.");
    process.exit(1);
  }
  const gitignorePath = path3.join(directoryPath, ".gitignore");
  if (!await fs3.pathExists(gitignorePath)) {
    logger.error(`Error: .gitignore file not found at ${gitignorePath}`);
    logger.error(
      "A .gitignore file is required to determine which files to process."
    );
    process.exit(1);
  }
  const casingType = cli.flags.casing;
  if (casingType !== "kebab" && casingType !== "snake") {
    logger.error("Error: Invalid casing type. Use 'kebab' or 'snake'.");
    process.exit(1);
  }
  const casingFn = getCasingFunction(casingType);
  logger.info("Rename phase 1/2...");
  await renameFilesAndFolders(directoryPath, gitignorePath, "phase1", casingFn);
  logger.info("Commit changes");
  await commitChanges("Rename files and folders phase 1/2");
  logger.info("Rename phase 2/2...");
  await renameFilesAndFolders(directoryPath, gitignorePath, "phase2", casingFn);
  logger.info("Commit changes");
  await commitChanges("Rename files and folders phase 2/2");
  logger.info("Running codemod to update import/export statements");
  await runCodemod(
    directoryPath,
    gitignorePath,
    casingType
  );
  logger.info("Commit changes");
  await commitChanges("Update import and export statements");
  logger.info("Conversion completed!");
}
run().catch((err) => {
  logger.error(err);
  process.exit(1);
});
